OUT_LOGGER = "ab-out"
OUT_LOGGER_FORMAT = "%(message)s"
TIMESTAMP_FORMAT = "%Y%m%d-%H%M%S%f"
TIMESTAMP_FORMAT_TOGETHER = "%Y%m%d%H%M%S%f"
NO_CACHE_TAG = "no-cache"

# configuration related constants
ANNOTATIONS_KEY = "annotations"
